@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2003 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -44,6 +44,10 @@ BEGIN
 	 postprocess_text =>
          { parse => 'code', type => CODEREF, optional => 1,
            descr => "A subroutine through which all plain text will be sent during compilation" },
+
+	 use_source_line_numbers =>
+	 { parse => 'boolean', type => SCALAR, default => 1,
+	   descr => "Whether to use source line numbers in errors and debugger" },
 	);
 
     __PACKAGE__->contained_objects
@@ -58,6 +62,7 @@ use HTML::Mason::MethodMaker
                           preprocess
                           postprocess_perl
                           postprocess_text
+			  use_source_line_numbers
                         )
 		    ],
     );
@@ -185,6 +190,9 @@ sub compile
 			  } );
     my $src = ref($p{comp_source}) ? $p{comp_source} : \$p{comp_source};
 
+    local $self->{current_compile} = {};
+    local $self->{paused_compiles} = []; # So we're re-entrant in subcomps
+
     # Preprocess the source.  The preprocessor routine is handed a
     # reference to the entire source.
     if ($self->preprocess)
@@ -201,18 +209,14 @@ sub compile
 sub start_component
 {
     my $self = shift;
+    my $c = $self->{current_compile};
 
-    compiler_error "Cannot start a component while already compiling a component"
-        if $self->{current_comp};
+    $c->{in_main} = 1;
+    $c->{comp_with_content_stack} = [];
 
-    $self->{in_main} = 1;
-    $self->{comp_with_content_stack} = [];
+    $c->{in_block} = undef;
 
-    $self->{in_block} = undef;
-
-    $self->_init_comp_data($self);
-
-    $self->{current_comp} = $self;
+    $self->_init_comp_data($c);
 }
 
 sub _init_comp_data
@@ -232,7 +236,7 @@ sub _init_comp_data
     $data->{flags} = {};
     $data->{attr} = {};
 
-    foreach ( qw( cleanup filter init once perl shared ) )
+    foreach ( qw( cleanup filter init once shared ) )
     {
 	$data->{blocks}{$_} = [];
     }
@@ -241,25 +245,25 @@ sub _init_comp_data
 sub end_component
 {
     my $self = shift;
+    my $c = $self->{current_compile};
 
     $self->lexer->throw_syntax_error("Not enough component-with-content ending tags found")
-	if $self->{comp_with_content_stack} && @{ $self->{comp_with_content_stack} };
-
-    $self->{current_comp} = undef;
+	if $c->{comp_with_content_stack} && @{ $c->{comp_with_content_stack} };
 }
 
 sub start_block
 {
     my $self = shift;
+    my $c = $self->{current_compile};
     my %p = @_;
 
     $self->lexer->throw_syntax_error("Cannot define a $p{block_type} section inside a method or subcomponent")
-	 if $top_level_only_block{ $p{block_type} } && ! $self->{in_main};
+	 if $top_level_only_block{ $p{block_type} } && ! $c->{in_main};
 
-    $self->lexer->throw_syntax_error("Cannot nest a $p{block_type} inside a $self->{in_block} block")
-	 if $self->{in_block};
+    $self->lexer->throw_syntax_error("Cannot nest a $p{block_type} inside a $c->{in_block} block")
+	 if $c->{in_block};
 
-    $self->{in_block} = $p{block_type};
+    $c->{in_block} = $p{block_type};
 }
 
 sub raw_block
@@ -267,6 +271,7 @@ sub raw_block
     # These blocks contain Perl code - so don't include <%text> and so on.
 
     my $self = shift;
+    my $c = $self->{current_compile};
     my %p = @_;
 
     eval { $self->postprocess_perl->( \$p{block} ) if $self->postprocess_perl };
@@ -280,10 +285,10 @@ sub raw_block
     {
 	my $line = $self->lexer->line_number;
 	my $file = $self->lexer->name;
-	$comment = "#line $line $file\n";
+	$comment = "#line $line $file\n" if $self->use_source_line_numbers;
     }
 
-    push @{ $self->{current_comp}{blocks}{ $p{block_type} } }, "$comment$p{block}";
+    push @{ $self->{current_compile}{blocks}{ $p{block_type} } }, "$comment$p{block}";
 }
 
 sub doc_block
@@ -298,7 +303,7 @@ sub perl_block
 
     $self->_add_body_code( $p{block} );
 
-    $self->{current_comp}{last_body_code_type} = 'perl_block';
+    $self->{current_compile}{last_body_code_type} = 'perl_block';
 }
 
 sub text
@@ -313,7 +318,7 @@ sub text
 
     $self->_add_body_code("\$m->print( '", $$tref, "' );\n");
 
-    $self->{current_comp}{last_body_code_type} = 'text';
+    $self->{current_compile}{last_body_code_type} = 'text';
 }
 
 sub text_block
@@ -326,12 +331,13 @@ sub text_block
 sub end_block
 {
     my $self = shift;
+    my $c = $self->{current_compile};
     my %p = @_;
 
-    $self->lexer->throw_syntax_error("End of $p{block_type} encountered while in $self->{in_block} block")
-	unless $self->{in_block} eq $p{block_type};
+    $self->lexer->throw_syntax_error("End of $p{block_type} encountered while in $c->{in_block} block")
+	unless $c->{in_block} eq $p{block_type};
 
-    $self->{in_block} = undef;
+    $c->{in_block} = undef;
 }
 
 sub variable_declaration
@@ -345,9 +351,9 @@ sub variable_declaration
     my $arg = "$p{type}$p{name}";
 
     $self->lexer->throw_syntax_error("$arg already defined")
-        if grep { "$_->{type}$_->{name}" eq $arg } @{ $self->{current_comp}{args} };
+        if grep { "$_->{type}$_->{name}" eq $arg } @{ $self->{current_compile}{args} };
 
-    push @{ $self->{current_comp}{args} }, { type => $p{type},
+    push @{ $self->{current_compile}{args} }, { type => $p{type},
 					     name => $p{name},
 					     default => $p{default},
 					     line => $self->lexer->line_number,
@@ -365,19 +371,20 @@ sub key_value_pair
 
     my $type = $p{block_type} eq 'flags' ? 'flag' : 'attribute';
     $self->lexer->throw_syntax_error("$p{key} $type already defined")
-	if exists $self->{current_comp}{ $p{block_type} }{ $p{key} };
+	if exists $self->{current_compile}{ $p{block_type} }{ $p{key} };
 
-    $self->{current_comp}{ $p{block_type} }{ $p{key} } = $p{value}
+    $self->{current_compile}{ $p{block_type} }{ $p{key} } = $p{value}
 }
 
 sub start_named_block
 {
     my $self = shift;
+    my $c = $self->{current_compile};
     my %p = @_;
 
     $self->lexer->throw_syntax_error
 	("Cannot define a $p{block_type} block inside a method or subcomponent")
-	    unless $self->{in_main};
+	    unless $c->{in_main};
 
     $self->lexer->throw_syntax_error("Invalid $p{block_type} name: $p{name}")
 	if $p{name} =~ /[^.\w-]/;
@@ -385,22 +392,22 @@ sub start_named_block
     my $other_type = $p{block_type} eq 'def' ? 'method' : 'def';
     $self->lexer->throw_syntax_error
         ("Cannot define a method and subcomponent with the same name ($p{name}")
-            if exists $self->{$other_type}{ $p{name} };
+            if exists $c->{$other_type}{ $p{name} };
 
-    $self->{in_main}--;
+    $c->{in_main}--;
 
-    $self->{ $p{block_type} }{ $p{name} } = {};
-    $self->_init_comp_data( $self->{ $p{block_type} }{ $p{name} } );
-    $self->{current_comp} = $self->{ $p{block_type} }{ $p{name} };
+    $c->{ $p{block_type} }{ $p{name} } = {};
+    $self->_init_comp_data( $c->{ $p{block_type} }{ $p{name} } );
+    push @{$self->{paused_compiles}}, $c;
+    $self->{current_compile} = $c->{ $p{block_type} }{ $p{name} };
 }
 
 sub end_named_block
 {
     my $self = shift;
 
-    $self->{in_main}++;
-
-    $self->{current_comp} = $self;
+    $self->{current_compile} = pop @{$self->{paused_compiles}};
+    $self->{current_compile}{in_main}++;
 }
 
 sub substitution
@@ -452,7 +459,7 @@ sub substitution
 
     $self->_add_body_code($code);
 
-    $self->{current_comp}{last_body_code_type} = 'substitution';
+    $self->{current_compile}{last_body_code_type} = 'substitution';
 }
 
 sub component_call
@@ -476,17 +483,18 @@ sub component_call
 
     $self->_add_body_code($code);
 
-    $self->{current_comp}{last_body_code_type} = 'component_call';
+    $self->{current_compile}{last_body_code_type} = 'component_call';
 }
 
 sub component_content_call
 {
     my $self = shift;
+    my $c = $self->{current_compile};
     my %p = @_;
 
     my $call = $p{call};
     for ($call) { s/^\s+//; s/\s+$//; }
-    push @{ $self->{comp_with_content_stack} }, $call;
+    push @{ $c->{comp_with_content_stack} }, $call;
 
     my $code = "\$m->comp( { content => sub {\n";
 
@@ -495,17 +503,18 @@ sub component_content_call
 
     $self->_add_body_code($code);
 
-    $self->{current_comp}{last_body_code_type} = 'component_content_call';
+    $c->{last_body_code_type} = 'component_content_call';
 }
 
 sub component_content_call_end
 {
     my $self = shift;
+    my $c = $self->{current_compile};
 
     $self->lexer->throw_syntax_error("found component with content ending tag but no beginning tag")
-	unless @{ $self->{comp_with_content_stack} };
+	unless @{ $c->{comp_with_content_stack} };
 
-    my $call = pop @{ $self->{comp_with_content_stack} };
+    my $call = pop @{ $c->{comp_with_content_stack} };
 
     if ( $call =~ m,^[\w/.],)
     {
@@ -522,7 +531,7 @@ sub component_content_call_end
 
     $self->_add_body_code($code);
 
-    $self->{current_comp}{last_body_code_type} = 'component_content_call_end';
+    $c->{last_body_code_type} = 'component_content_call_end';
 }
 
 sub perl_line
@@ -537,7 +546,7 @@ sub perl_line
 
     $self->_add_body_code($code);
 
-    $self->{current_comp}{last_body_code_type} = 'perl_line';
+    $self->{current_compile}{last_body_code_type} = 'perl_line';
 }
 
 sub _add_body_code
@@ -550,34 +559,35 @@ sub _add_body_code
     # can break certain constructs like qw() list that spans multiple
     # perl-lines.
     if ( $self->lexer->line_number &&
-         $self->{current_comp}{last_body_code_type} ne 'perl_line' )
+         $self->{current_compile}{last_body_code_type} ne 'perl_line' )
     {
 	my $line = $self->lexer->line_number;
 	my $file = $self->lexer->name;
-	$self->{current_comp}{body} .= "#line $line $file\n";
+	$self->{current_compile}{body} .= "#line $line $file\n" if $self->use_source_line_numbers;
     }
 
-    $self->{current_comp}{body} .= $_ foreach @_;
+    $self->{current_compile}{body} .= $_ foreach @_;
 }
 
 sub dump
 {
     my $self = shift;
+    my $c = $self->{current_compile};
 
     warn "Main component\n";
 
-    $self->_dump_data( $self );
+    $self->_dump_data( $c );
 
-    foreach ( keys %{ $self->{def} } )
+    foreach ( keys %{ $c->{def} } )
     {
 	warn "  Subcomponent $_\n";
-	$self->_dump_data( $self->{def}{$_}, '  ' );
+	$self->_dump_data( $c->{def}{$_}, '  ' );
     }
 
-    foreach ( keys %{ $self->{method} } )
+    foreach ( keys %{ $c->{method} } )
     {
 	warn "  Methods $_\n";
-	$self->_dump_data( $self->{method}{$_}, '  ');
+	$self->_dump_data( $c->{method}{$_}, '  ');
     }
 }
 
@@ -606,7 +616,7 @@ sub _blocks
 {
     my $self = shift;
 
-    return @{ $self->{current_comp}{blocks}{ shift() } };
+    return @{ $self->{current_compile}{blocks}{ shift() } };
 }
 
 sub HTML::Mason::Parser::new
@@ -702,6 +712,15 @@ reference to the Perl portion of the component.  The sub is expected
 to process the string in-place. See also
 L<preprocess|HTML::Mason::Params/preprocess> and L<postprocess_text|HTML::Mason::Params/postprocess_text>.
 
+=item use_source_line_numbers
+
+True or false, default is true. Indicates whether component line
+numbers that appear in error messages, stack traces, etc. are in terms
+of the source file instead of the object file. Mason does this by
+inserting '#line' directives into compiled components.  While source
+line numbers are more immediately helpful, object file line numbers
+may be more appropriate for in-depth debugging sessions.
+
 =back
 
 =head1 METHODS
